﻿namespace AirplaneWithRadar
{
    // Карта с набром объектов под нее
    internal class MapWithSetAirplaneGeneric<T, U>
        where T : class, IDrawningObject, IEquatable<T>
        where U : AbstractMap
    {
        // Ширина окна отрисовки
        private readonly int _pictureWidth;
        // Высота окна отрисовки
        private readonly int _pictureHeight;
        // Размер занимаемого объектом места (ширина)
        private readonly int _placeSizeWidth = 120;
        // Размер занимаемого объектом места (высота)
        private readonly int _placeSizeHeight = 80;
        // Набор объектов
        private readonly SetAirplaneGeneric<T> _setAirplane;
        // Карта
        private readonly U _map;
        // Конструктор
        public MapWithSetAirplaneGeneric(int picWidth, int picHeight, U map)
        {
            int width = picWidth / _placeSizeWidth;
            int height = picHeight / _placeSizeHeight;
            _setAirplane = new SetAirplaneGeneric<T>(width * height);
            _pictureWidth = picWidth;
            _pictureHeight = picHeight;
            _map = map;
        }
        // Перегрузка оператора сложения
        public static int operator +(MapWithSetAirplaneGeneric<T, U> map, T airplane)
        {
            return map._setAirplane.Insert(airplane);
        }
        // Перегрузка оператора вычитания
        public static T operator -(MapWithSetAirplaneGeneric<T, U> map, int position)
        {
            return map._setAirplane.Remove(position);
        }
        // Вывод всего набора объектов
        public Bitmap ShowSet()
        {
            Bitmap bmp = new(_pictureWidth, _pictureHeight);
            Graphics gr = Graphics.FromImage(bmp);
            DrawBackground(gr);
            DrawAirplane(gr);
            return bmp;
        }
        // Просмотр объекта на карте
        public Bitmap ShowOnMap()
        {
            Shaking();
            foreach (var ship in _setAirplane.GetAirplane())
            {
                return _map.CreateMap(_pictureWidth, _pictureHeight, ship);
            }
            return new(_pictureWidth, _pictureHeight);
        }
        // Перемещение объекта по карте
        public Bitmap MoveObject(Direction direction)
        {
            if (_map != null)
            {
                return _map.MoveObject(direction);
            }
            return new(_pictureWidth, _pictureHeight);
        }
        /// <summary>
        /// Получение данных в виде строки
        /// </summary>
        /// <param name="sep"></param>
        /// <returns></returns>
        public string GetData(char separatorType, char separatorData)
        {
            string data = $"{_map.GetType().Name}{separatorType}";
            foreach (var airplane in _setAirplane.GetAirplane())
            {
                data += $"{airplane.GetInfo()}{separatorData}";
            }
            return data;
        }
        /// <summary>
        /// Загрузка списка из массива строк
        /// </summary>
        /// <param name="records"></param>
        public void LoadData(string[] records)
        {
            foreach (var rec in records)
            {
                _setAirplane.Insert(DrawningObjectAirplane.Create(rec) as T);
            }
        }
        /// <summary>
        /// Сортировка
        /// </summary>
        /// <param name="comparer"></param>
        public void Sort(IComparer<T> comparer)
        {
            _setAirplane.SortSet(comparer);
        }
        // "Взбалтываем" набор, чтобы все элементы оказались в начале
        private void Shaking()
        {
            int j = _setAirplane.Count - 1;
            for (int i = 0; i < _setAirplane.Count; i++)
            {
                if (_setAirplane[i] == null)
                {
                    for (; j > i; j--)
                    {
                        var airplane = _setAirplane[j];
                        if (airplane != null)
                        {
                            _setAirplane.Insert(airplane, i);
                            _setAirplane.Remove(j);
                            break;
                        }
                    }
                    if (j <= i)
                    {
                        return;
                    }
                }
            }
        }
        // Метод отрисовки фона
        private void DrawBackground(Graphics g)
        {
            Pen pen = new(Color.Black, 3);
            Brush brush = new SolidBrush(Color.FromArgb(147, 146, 151));
            g.FillRectangle(brush, 0, 0, _pictureWidth, _pictureHeight);
            for (int i = 0; i <= _pictureWidth / _placeSizeWidth; i++)
            {
                for (int j = 0; j <= (_pictureHeight / _placeSizeHeight); ++j)
                {//линия разметки места
                    g.DrawLine(pen, i * _placeSizeWidth, j * _placeSizeHeight, i * _placeSizeWidth + _placeSizeWidth, j * _placeSizeHeight);
                    g.DrawLine(pen, i * _placeSizeWidth, j * _placeSizeHeight + 10, i * _placeSizeWidth + _placeSizeWidth, j * _placeSizeHeight + 10);
                }
                g.DrawLine(pen, i * _placeSizeWidth, 0, i * _placeSizeWidth, (_pictureHeight / _placeSizeHeight) * _placeSizeHeight);
            }
        }
        // Метод прорисовки объектов
        private void DrawAirplane(Graphics g)
        {
            int widthEl = _pictureWidth / _placeSizeWidth;
            int heightEl = _pictureHeight / _placeSizeHeight;

            int curWidth = 0;
            int curHeight = 0;

            foreach (var airplane in _setAirplane.GetAirplane())
            {
                airplane?.SetObject(_pictureWidth - _placeSizeWidth * curWidth - 80,
                    curHeight * _placeSizeHeight + 30, _pictureWidth, _pictureHeight);
                airplane?.DrawningObject(g);

                if (curWidth < widthEl - 1)
                    curWidth++;
                else
                {
                    curWidth = 0;
                    curHeight++;
                }
                if (curHeight > heightEl)
                {
                    return;
                }
            }
        }
    }
}